// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"property-management-gateway/errors"
	param_v1 "property-management-gateway/param/v1"
	"property-management-gateway/pb"
	"property-management-gateway/pb/v1"
)

//
// @Summary 添加应用
// @Description 添加应用
// @Tags 应用
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ApplicationAddBody true "信息"
// @Success 200 {object} v1.ApplicationAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/application [post]
func (c *Controller) ApplicationAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationAddRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApplicationAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationAddResponse{}
		rpcReq := &v1.ApplicationAddRequest{
			Name:    req.Name,
			Content: req.Content,
			Desc:    req.Desc,
			Icon:    req.Icon,
			Price:   req.Price,
		}

		_, err := pb.Common.ApplicationAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 编辑应用
// @Description 编辑应用
// @Tags 应用
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ApplicationUpdateBody true "信息"
// @Success 200 {object} v1.ApplicationUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/application [put]
func (c *Controller) ApplicationUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApplicationUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationUpdateResponse{}
		rpcReq := &v1.ApplicationUpdateRequest{
			Name:    req.Name,
			Content: req.Content,
			Desc:    req.Desc,
			Icon:    req.Icon,
			Price:   req.Price,
			Id:      req.Id,
		}

		_, err := pb.Common.ApplicationUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 发布和下架
// @Description 发布和下架
// @Tags 应用
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ApplicationEnableSetBody true "信息"
// @Success 200 {object} v1.ApplicationEnableSetResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/application/enable [put]
func (c *Controller) ApplicationEnableSet(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationEnableSetRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApplicationEnableSetBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationEnableSetResponse{}
		rpcReq := &v1.ApplicationEnableSetRequest{
			Enable: req.Enable,
			Id:     req.Id,
		}

		_, err := pb.Common.ApplicationEnableSet(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationEnableSet"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 给应用添加菜单
// @Description 给应用添加菜单
// @Tags 应用
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ApplicationPermissionsSetBody true "信息"
// @Success 200 {object} v1.ApplicationPermissionsSetResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/application/permissions [put]
func (c *Controller) ApplicationPermissionsSet(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationPermissionsSetRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApplicationPermissionsSetBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationPermissionsSetResponse{}
		rpcReq := &v1.ApplicationPermissionsSetRequest{
			Permissions: req.Permissions,
			Id:          req.Id,
		}

		_, err := pb.Common.ApplicationPermissionsSet(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationPermissionsSet"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除应用
// @Description 删除应用
// @Tags 应用
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true "应用id"
// @Success 200 {object} v1.ApplicationDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/application [delete]
func (c *Controller) ApplicationDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ApplicationDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationDelResponse{}
		rpcReq := &v1.ApplicationDelRequest{
			Id: req.Id,
		}

		_, err := pb.Common.ApplicationDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 应用列表
// @Description 应用列表
// @Tags 应用
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true "第几页，1为起始页, -1 不分页返回所有"
// @Param page_size query int false "每页条数,-1 不分页返回所有"
// @Param enable query int false "0不过滤,1已发布的 2 未发布的"
// @Success 200 {object} v1.ApplicationListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/application [get]
func (c *Controller) ApplicationList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ApplicationListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationListResponse{}
		rpcReq := &v1.ApplicationListRequest{
			//PackageId: req.PackageId,
			PageSize: req.PageSize,
			Page:     req.Page,
			Enable:   req.Enable,
		}

		rpcRsp, err := pb.Common.ApplicationList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.ApplicationItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

func fixEmptyApplicationPermissionItem(list []*v1.ApplicationPermissionItem) []*v1.ApplicationPermissionItem {
	if list == nil {
		return make([]*v1.ApplicationPermissionItem, 0)
	}
	for i, v := range list {
		if v.Childs == nil {
			list[i].Childs = make([]*v1.ApplicationPermissionItem, 0)
			continue
		}
		list[i].Childs = fixEmptyApplicationPermissionItem(v.Childs)
	}
	return list
}

//
// @Summary 应用的菜单列表
// @Description 应用的菜单列表
// @Tags 应用
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true "应用id"
// @Success 200 {object} v1.ApplicationPermissionsResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/application/permissions [get]
func (c *Controller) ApplicationPermissions(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationPermissionsRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ApplicationPermissionsQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationPermissionsResponse{}
		rpcReq := &v1.ApplicationPermissionsRequest{
			//PackageId: req.PackageId,
			Id: req.Id,
		}

		rpcRsp, err := pb.Common.ApplicationPermissions(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationPermissions"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		rpcRsp.List = fixEmptyApplicationPermissionItem(rpcRsp.List)
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
