// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"property-management-gateway/errors"
	param_v1 "property-management-gateway/param/v1"
	"property-management-gateway/pb"
	"property-management-gateway/pb/v1"
	"property-management-gateway/utils"
)

//
// @Summary 添加公司
// @Description 添加公司
// @Tags 物业公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.CompanyRegisterBody true "注册信息"
// @Success 200 {object} v1.CompanyRegisterResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/register [post]
func (c *Controller) CompanyRegister(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyRegisterRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.CompanyRegisterBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.CompanyRegisterResponse{}
		rpcReq := &v1.RegisterRequest{
			Applicant:               req.Applicant,
			ApplicantPhone:          req.ApplicantPhone,
			CompanyName:             req.CompanyName,
			CompanyPhone:            req.CompanyPhone,
			ApplicantDuty:           req.ApplicantDuty,
			LegalPerson:             req.LegalPerson,
			CompanyAddr:             req.CompanyAddr,
			SocialCode:              req.SocialCode,
			BusinessLicense:         req.BusinessLicense,
			ApplicantIdentification: req.ApplicantIdentification,
			LicenseType:             req.LicenseType,
			FreeGardenCount:         req.FreeGardenCount,
			Username:                req.Username,
			Password:                req.Password,
			//Expire:req.Expire,
			IsByManager: true,
		}

		rpcRsp, err := pb.Company.Register(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.Register"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:   ModuleCompany,
			Action:   ActionCompanyAdd,
			Origin:   nil,
			Target:   req.CompanyRegisterBody,
			UserName: tokenInfo.User,
			Uid:      tokenInfo.Uid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 审核公司
// @Description 审核公司
// @Tags 物业公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.CompanyApproveBody true "审批信息"
// @Success 200 {object} v1.CompanyApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/approve [put]
func (c *Controller) CompanyApprove(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.CompanyApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.CompanyApproveResponse{}
		rpcReq := &v1.CompanyApproveRequest{
			Id:       req.Id,
			Status:   req.Status,
			Feedback: req.Feedback,
		}

		_, err := pb.Company.CompanyApprove(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 公司列表
// @Description 公司列表
// @Tags 物业公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param company_name query string false "公司名称"
// @Param approve_status query int false "0 所有 1 待审核 2 未通过 3 通过"
// @Param page query int true "第几页，1为起始页, -1 不分页返回所有"
// @Param page_size query int false "每页条数"
// @Success 200 {object} v1.CompanyListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/list [get]
func (c *Controller) CompanyList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.CompanyListResponse{}
		rpcReq := &v1.CompanyListRequest{
			CompanyName:   req.CompanyName,
			PageSize:      req.PageSize,
			Page:          req.Page,
			ApproveStatus: req.ApproveStatus,
		}

		rpcRsp, err := pb.Company.CompanyList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.CompanyItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 修改配额
// @Description 修改配额
// @Tags 物业公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.CompanyChangeFreeGardenBody true " "
// @Success 200 {object} v1.CompanyChangeFreeGardenResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/free_garden [put]
func (c *Controller) CompanyChangeFreeGarden(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyChangeFreeGardenRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.CompanyChangeFreeGardenBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.CompanyChangeFreeGardenResponse{}
		rpcReq := &v1.CompanyChangeFreeGardenRequest{
			Id:    req.Id,
			Count: req.Count,
		}

		_, err := pb.Company.CompanyChangeFreeGarden(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyChangeFreeGarden"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  微信商户申请记录
// @Description 微信商户申请记录
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true " "
// @Param page_size query int true " "
// @Success 200 {object} v1.CompanyWxAccountListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/wx_account [get]
func (c *Controller) CompanyWxAccountList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyWxAccountListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.CompanyWxAccountListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.CompanyWxAccountListResponse{}
		rpcReq := &v1.CompanyWxAccountApplyListRequest{
			PageSize:   req.PageSize,
			Page:       req.Page,
			Management: true,
		}

		rpcRsp, err := pb.Company.CompanyWxAccountApplyList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyWxAccountList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary  微信商户申请资料详情
// @Description 微信商户申请资料详情
// @Tags 公司
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true "申请记录id"
// @Success 200 {object} v1.CompanyWxAccountApplyInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/wx_account_info [get]
func (c *Controller) CompanyWxAccountApplyInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyWxAccountApplyInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.CompanyWxAccountApplyInfoQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.CompanyWxAccountApplyInfoResponse{}
		rpcReq := &v1.CompanyWxAccountApplyInfoRequest{
			Id:         req.Id,
			Management: true,
		}

		rpcRsp, err := pb.Company.CompanyWxAccountApplyInfo(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Company.CompanyWxAccountApplyInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.SubjectInfo.UboInfos == nil {
			rpcRsp.SubjectInfo.UboInfos = make([]*v1.CompanyWxIdDocInfo, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
