package v1

import (
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"github.com/tealeg/xlsx"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"net/http"
	"os"
	"path"
	"property-management-gateway/errors"
	param_v1 "property-management-gateway/param/v1"
	"property-management-gateway/parser"
	"property-management-gateway/pb"
	"property-management-gateway/pb/v1"
	"property-management-gateway/utils"
	"strconv"
	"strings"
	"time"
)

//
// @Summary 门禁协议列表
// @Description 门禁协议列表
// @Tags 门禁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.GateProtocolsResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/gate/protocols [get]
func (c *Controller) GateProtocols(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GateProtocolsRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		resp := param_v1.GateProtocolsResponse{}
		rpcReq := &v1.GateProtocolsRequest{}
		rpcRsp, err := pb.Device.GateProtocols(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "Device.GateProtocols"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		resp.Data = *rpcRsp

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 门禁列表
// @Description 门禁列表
// @Tags 门禁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int false " "
// @Param page_size query int false " "
// @Param garden_id query int false "小区id"
// @Param out query int false "0 不过滤 1 已出库 2 未出库"
// @Param sn query string false "设备sn"
// @Success 200 {object} v1.GateListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/gate [get]
func (c *Controller) GateList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GateListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.GateListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		resp := param_v1.GateListResponse{}
		rpcReq := &v1.GateListRequest{
			Page:     req.Page,
			PageSize: req.PageSize,
			GardenId: req.GardenId,
			Out:      req.Out,
			Sn:       req.Sn,
		}
		rpcRsp, err := pb.Device.GateList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "Device.GateList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.GateItem, 0)
		}
		resp.Data = *rpcRsp

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 门禁入库
// @Description 门禁入库
// @Tags 门禁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.GateInBody true " "
// @Success 200 {object} v1.GateInResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/gate/in [post]
func (c *Controller) GateIn(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GateInRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.GateInBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//return err
		//}
		resp := param_v1.GateInResponse{}
		rpcReq := &v1.GateInRequest{
			// 设备名
			DeviceName: req.DeviceName,
			// 序列号
			Sn: req.Sn,
			// 厂商
			Manufactor: req.Manufactor,
			// 授权key
			AuthKey: req.AuthKey,

			// 协议
			Protocol: req.Protocol,
		}

		_, err := pb.Device.GateIn(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Device.GateIn"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

func parseGateImportContent(fileName string) ([]*v1.GateBatchInItem, error) {
	f, err := xlsx.OpenFile(fileName)
	if err != nil {
		return nil, status.Error(10003, "xlsx 打开失败")
	}

	if len(f.Sheets) == 0 {
		return nil, status.Error(10003, "参数文件没有sheet")
	}

	sheet := f.Sheets[0]
	if len(sheet.Rows) == 0 {
		return nil, status.Error(10003, "文件为空")
	}

	if len(sheet.Rows) > 1005 {
		return nil, status.Error(10003, "单次导入不能超过一千条记录, 请分批导入")
	}

	list := []*v1.GateBatchInItem{}
	for i, row := range sheet.Rows {
		if len(row.Cells) < 5 {
			return nil, status.Error(10003, fmt.Sprintf("第%d行", i)+"列数错误，请使用模板")
		}
		if i == 0 {
			if row.Cells[0].Value != "设备名" ||
				row.Cells[1].Value != "设备序列号" ||
				row.Cells[2].Value != "厂商" ||
				row.Cells[3].Value != "授权key" ||
				row.Cells[4].Value != "协议" {
				return nil, status.Error(10003, fmt.Sprintf("第%d行", i+1)+"标题错误，请使用模板")
			}
			continue
		}

		//deviceId := strings.TrimSpace(row.Cells[0].Value)
		deviceName := strings.TrimSpace(row.Cells[0].Value)
		deviceSn := strings.TrimSpace(row.Cells[1].Value)
		manufactor := strings.TrimSpace(row.Cells[2].Value)
		key := strings.TrimSpace(row.Cells[3].Value)
		protocolStr := strings.TrimSpace(row.Cells[5].Value)
		protocol, _ := strconv.Atoi(protocolStr)

		if deviceName == "" &&
			deviceSn == "" && manufactor == "" && key == "" && protocol == 0 {
			continue
		}
		if deviceSn == "" || manufactor == "" || key == "" || protocol == 0 {
			return nil, status.Error(10003, fmt.Sprintf("第%d行", i+1)+"参数不能为空")
		}

		item := &v1.GateBatchInItem{
			//DeviceId:   deviceId,
			DeviceName: deviceName,
			Sn:         deviceSn,
			Manufactor: manufactor,
			AuthKey:    key,
			Protocol:   int32(protocol),
		}
		list = append(list, item)
	}

	return list, nil
}

//
// @Summary 获取门禁模板
// @Description 获取门禁模板
// @Tags 门禁
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Success 200 {object} v1.TemplateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/gate/template [get]
func (c *Controller) GateTemplate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.TemplateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.TemplateResponse{}
		resp.Data.Url = parser.Conf.Oss.GateTempUrl
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 门禁批量入库
// @Description 门禁批量入库
// @Tags 门禁
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param file formData file true "file"
// @Success 200 {object} v1.GateInResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/gate/import [post]
func (c Controller) GateImport(ctx *gin.Context) {
	// 解析参数
	newFile := ""

	var tokenInfo utils.TokenInfo
	parseParamTask := func() error {
		var err error
		tokenInfo, err = utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		file, err := ctx.FormFile("file")
		if err != nil {
			return errors.SystemError
		}

		if file.Size > 10*1024*1024 {
			return status.Error(10003, "文件过大")
		}
		ext := path.Ext(file.Filename)
		if ext != ".xlsx" {
			return status.Error(10003, "仅支持xlsx文件格式")
		}
		newFile = fmt.Sprintf("power-%s-%d-%d.xlsx", file.Filename, tokenInfo.Uid, time.Now().UnixNano())
		err = ctx.SaveUploadedFile(file, newFile)
		if err != nil {
			logger.Error("func",
				zap.String("call", "ctx.SaveUploadedFile"),
				zap.String("error", err.Error()))
			return status.Error(10003, "保存文件失败")
		}

		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		list, err := parseGateImportContent(newFile)

		if err != nil {
			os.Remove(newFile)
			return err
		}
		os.Remove(newFile)
		if len(list) == 0 {
			return nil
		}
		// 响应数据
		resp := param_v1.GateInResponse{}
		rpcReq := &v1.GateBatchInRequest{
			List: list,
		}

		_, err = pb.Device.GateBatchIn(ctx, rpcReq)
		if err != nil {
			logger.Error("func",
				zap.String("call", "Device.GateBatchIn"),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 门禁出库
// @Description 门禁出库
// @Tags 门禁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.GateOutBody true " "
// @Success 200 {object} v1.GateOutResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/gate/out [put]
func (c *Controller) GateOut(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GateOutRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.GateOutBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.GateOutResponse{}
		rpcReq := &v1.GateOutRequest{
			DeviceId: req.DeviceId,
			GardenId: req.GardenId,
			OutUser:  tokenInfo.User,
		}

		_, err = pb.Device.GateOut(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Device.GateOut"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 门禁回收
// @Description 门禁回收
// @Tags 门禁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.GateRecoveryBody true " "
// @Success 200 {object} v1.GateRecoveryResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/gate/recovery [put]
func (c *Controller) GateRecovery(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GateRecoveryRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.GateRecoveryBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//return err
		//}
		resp := param_v1.GateRecoveryResponse{}
		rpcReq := &v1.GateRecoveryRequest{
			DeviceId: req.DeviceId,
		}

		_, err := pb.Device.GateRecovery(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Device.GateRecovery"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 门禁删除
// @Description 门禁删除
// @Tags 门禁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param device_id query int64 true "设备id"
// @Success 200 {object} v1.GateDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/gate [delete]
func (c *Controller) GateDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GateDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.GateDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//return err
		//}
		resp := param_v1.GateDelResponse{}
		rpcReq := &v1.GateDelRequest{
			DeviceId: req.DeviceId,
		}

		_, err := pb.Device.GateDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Device.GateDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
