// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"property-management-gateway/errors"
	param_v1 "property-management-gateway/param/v1"
	"property-management-gateway/pb"
	"property-management-gateway/pb/v1"
)

//
// @Summary 添加素材
// @Description 添加素材
// @Tags 物业素材
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.MaterialAddBody true "信息"
// @Success 200 {object} v1.MaterialAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/material [post]
func (c *Controller) MaterialAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MaterialAddRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.MaterialAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.MaterialAddResponse{}
		rpcReq := &v1.MaterialAddRequest{
			Mtype: req.Mtype,
			Pic:   req.Pic,
		}

		_, err := pb.Common.MaterialAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.MaterialAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 更改素材
// @Description 更改素材
// @Tags 物业素材
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.MaterialUpdateBody true "信息"
// @Success 200 {object} v1.MaterialUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/material [put]
func (c *Controller) MaterialUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MaterialUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.MaterialUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.MaterialUpdateResponse{}
		rpcReq := &v1.MaterialUpdateRequest{
			Id:  req.Id,
			Pic: req.Pic,
		}

		_, err := pb.Common.MaterialUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.MaterialUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除素材
// @Description 删除素材
// @Tags 物业素材
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true " "
// @Success 200 {object} v1.MaterialDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/material [delete]
func (c *Controller) MaterialDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MaterialDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.MaterialDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.MaterialDelResponse{}
		rpcReq := &v1.MaterialDelRequest{
			Id: req.Id,
		}

		_, err := pb.Common.MaterialDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.MaterialDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 素材列表
// @Description 素材列表
// @Tags 物业素材
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int false " "
// @Param page_size query int false " "
// @Success 200 {object} v1.MaterialListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/material [get]
func (c *Controller) MaterialList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MaterialListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.MaterialListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.MaterialListResponse{}
		rpcReq := &v1.MaterialListRequest{
			PageSize: req.PageSize,
			Page:     req.Page,
		}

		rpcRsp, err := pb.Common.MaterialList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.MaterialList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.MaterialItem, 0)
		}

		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 发布或下架
// @Description 发布或下架
// @Tags 物业素材
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.MaterialEnableBody true " "
// @Success 200 {object} v1.MaterialUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/material/enable [put]
func (c *Controller) MaterialEnable(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MaterialEnableRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.MaterialEnableBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.MaterialEnableResponse{}
		rpcReq := &v1.MaterialEnableRequest{
			Id:     req.Id,
			Enable: req.Enable,
		}
		_, err := pb.Common.MaterialEnable(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.MaterialEnable"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
