// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"property-management-gateway/errors"
	param_v1 "property-management-gateway/param/v1"
	"property-management-gateway/pb"
	"property-management-gateway/pb/v1"
)

//
// @Summary 添加套餐
// @Description 添加套餐
// @Tags 短信套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.MsgPackageAddBody true "信息"
// @Success 200 {object} v1.MsgPackageAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/msg_package [post]
func (c *Controller) MsgPackageAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MsgPackageAddRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.MsgPackageAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.MsgPackageAddResponse{}
		rpcReq := &v1.MsgPackageAddRequest{
			Name:     req.Name,
			MsgCount: req.MsgCount,
			Comment:  req.Comment,
			Price:    req.Price,
		}

		_, err := pb.Common.MsgPackageAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.MsgPackageAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 编辑套餐
// @Description 编辑套餐
// @Tags 短信套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.MsgPackageUpdateBody true "信息"
// @Success 200 {object} v1.MsgPackageUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/msg_package [put]
func (c *Controller) MsgPackageUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MsgPackageUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.MsgPackageUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.MsgPackageUpdateResponse{}
		rpcReq := &v1.MsgPackageUpdateRequest{
			Name:     req.Name,
			Comment:  req.Comment,
			MsgCount: req.MsgCount,
			Price:    req.Price,
			Id:       req.Id,
		}

		_, err := pb.Common.MsgPackageUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.MsgPackageUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除套餐
// @Description 删除套餐
// @Tags 短信套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true "套餐id"
// @Success 200 {object} v1.MsgPackageDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/msg_package [delete]
func (c *Controller) MsgPackageDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MsgPackageDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.MsgPackageDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.MsgPackageDelResponse{}
		rpcReq := &v1.MsgPackageDelRequest{
			Id: req.Id,
		}

		_, err := pb.Common.MsgPackageDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.MsgPackageDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 套餐列表
// @Description 套餐列表
// @Tags 短信套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true "第几页，1为起始页, -1 不分页返回所有"
// @Param page_size query int false "每页条数,-1 不分页返回所有"
// @Success 200 {object} v1.MsgPackageListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/msg_package [get]
func (c *Controller) MsgPackageList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MsgPackageListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.MsgPackageListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.MsgPackageListResponse{}
		rpcReq := &v1.MsgPackageListRequest{
			//MsgPackageId: req.MsgPackageId,
			PageSize: req.PageSize,
			Page:     req.Page,
		}

		rpcRsp, err := pb.Common.MsgPackageList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.MsgPackageList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.MsgPackageItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
