// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"property-management-gateway/errors"
	param_v1 "property-management-gateway/param/v1"
	"property-management-gateway/pb"
	"property-management-gateway/pb/v1"
)

//
// @Summary 应用订单审批
// @Description 应用订单审批
// @Tags 应用订单
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ApplicationOrderApproveBody true "信息"
// @Success 200 {object} v1.ApplicationOrderApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/application [put]
func (c *Controller) ApplicationOrderApprove(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationOrderApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApplicationOrderApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationOrderApproveResponse{}
		rpcReq := &v1.ApplicationOrderApproveRequest{
			OrderId:  req.OrderId,
			Status:   req.Status,
			Feedback: req.Feedback,
		}

		_, err := pb.System.ApplicationOrderApprove(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.ApplicationOrderApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 应用订单列表
// @Description 应用订单列表
// @Tags 应用订单
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true "第几页，1为起始页, -1 不分页返回所有"
// @Param page_size query int false "每页条数,-1 不分页返回所有"
// @Param status query int false "0不过滤,1待认证 2 已通过 3 未通过"
// @Success 200 {object} v1.GardenApplicationListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/application [get]
func (c *Controller) GardenApplicationList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GardenApplicationListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.GardenApplicationListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.GardenApplicationListResponse{}
		rpcReq := &v1.GardenApplicationListRequest{
			Status:   req.Status,
			PageSize: req.PageSize,
			Page:     req.Page,
		}

		rpcRsp, err := pb.System.GardenApplicationList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.GardenApplicationList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.GardenApplicationItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 套餐订单审批
// @Description 套餐订单审批
// @Tags 套餐订单
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.PackageOrderApproveBody true "信息"
// @Success 200 {object} v1.PackageOrderApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/package [put]
func (c *Controller) PackageOrderApprove(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageOrderApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.PackageOrderApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageOrderApproveResponse{}
		rpcReq := &v1.PackageOrderApproveRequest{
			OrderId:  req.OrderId,
			Status:   req.Status,
			Feedback: req.Feedback,
		}

		_, err := pb.System.PackageOrderApprove(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.PackageOrderApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 套餐订单改价
// @Description 套餐订单改价
// @Tags 套餐订单
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.PackageOrderAmountBody true "信息"
// @Success 200 {object} v1.PackageOrderAmountResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/package/amount [put]
func (c *Controller) PackageOrderAmount(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageOrderAmountRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.PackageOrderAmountBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageOrderAmountResponse{}
		rpcReq := &v1.PackageOrderAmountRequest{
			OrderId: req.OrderId,
			Amount:  req.Amount,
		}

		_, err := pb.System.PackageOrderAmount(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.PackageOrderAmount"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 应用订单改价
// @Description 应用订单改价
// @Tags 套餐订单
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ApplicationOrderAmountBody true "信息"
// @Success 200 {object} v1.ApplicationOrderAmountResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/application/amount [put]
func (c *Controller) ApplicationOrderAmount(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationOrderAmountRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApplicationOrderAmountBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationOrderAmountResponse{}
		rpcReq := &v1.ApplicationOrderAmountRequest{
			OrderId: req.OrderId,
			Amount:  req.Amount,
		}

		_, err := pb.System.ApplicationOrderAmount(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.ApplicationOrderAmount"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 套餐订单列表
// @Description 套餐订单列表
// @Tags 套餐订单
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true "第几页，1为起始页, -1 不分页返回所有"
// @Param page_size query int false "每页条数,-1 不分页返回所有"
// @Param status query int false "0不过滤,1待认证 2 已通过 3 未通过"
// @Param cid query int false "公司id（用于获取公司下的套餐）"
// @Success 200 {object} v1.CompanyPackageListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/package [get]
func (c *Controller) CompanyPackageList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyPackageListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyPackageListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.CompanyPackageListResponse{}
		rpcReq := &v1.CompanyPackageListRequest{
			Status:     req.Status,
			PageSize:   req.PageSize,
			Page:       req.Page,
			Cid:        req.Cid,
			Management: true,
		}

		rpcRsp, err := pb.System.CompanyPackageList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.CompanyPackageList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.CompanyPackageItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 新增套餐
// @Description 新增套餐
// @Tags 公司套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.PackageOrderAddBody true "信息"
// @Success 200 {object} v1.PackageOrderAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/package [post]
func (c *Controller) PackageOrderAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageOrderAddRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.PackageOrderAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageOrderAddResponse{}
		rpcReq := &v1.PackageOrderAddRequest{
			Cid:          req.Cid,
			PackageId:    req.PackageId,
			Period:       req.Period,
			GardenCount:  1,
			Amount:       req.Amount,
			CompanyName:  req.CompanyName,
			OrderType:    1,
			ShouldAccess: true,
		}

		_, err := pb.System.PackageOrderAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.PackageOrderAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 套餐续费
// @Description 套餐续费
// @Tags 公司套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.PackageOrderAddRenewBody true "信息"
// @Success 200 {object} v1.PackageOrderAddRenewResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/renew [post]
func (c *Controller) PackageOrderAddRenew(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageOrderAddRenewRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.PackageOrderAddRenewBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageOrderAddRenewResponse{}
		rpcReq := &v1.PackageOrderAddRequest{
			PackageId:    req.PackageId,
			Period:       req.Period,
			GardenCount:  1,
			Amount:       req.Amount,
			GardenId:     req.GardenId,
			OrderType:    2,
			ShouldAccess: true,
		}

		_, err := pb.System.PackageOrderAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.PackageOrderAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 套餐列表
// @Description 套餐列表
// @Tags 公司套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true "第几页，1为起始页, -1 不分页返回所有"
// @Param page_size query int false "每页条数,-1 不分页返回所有"
// @Param cid query int false "公司id（用于获取公司下的套餐）"
// @Success 200 {object} v1.CompanyPackageListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/package [get]
func (c *Controller) CompanyPackage(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyPackageListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyPackageListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.CompanyPackageListResponse{}
		rpcReq := &v1.CompanyPackageListRequest{
			Status:    2,
			PageSize:  req.PageSize,
			Page:      req.Page,
			Cid:       req.Cid,
			OrderType: 1,
		}

		rpcRsp, err := pb.System.CompanyPackageList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.CompanyPackageList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.CompanyPackageItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// @Summary 短信订单审批
// @Description 短信订单审批
// @Tags 短信订单
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.MsgPackageOrderApproveBody true "信息"
// @Success 200 {object} v1.MsgPackageOrderApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/msg_package [put]
func (c *Controller) MsgPackageOrderApprove(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MsgPackageOrderApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.MsgPackageOrderApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.MsgPackageOrderApproveResponse{}
		rpcReq := &v1.MsgPackageOrderApproveRequest{
			OrderId:  req.OrderId,
			Status:   req.Status,
			Feedback: req.Feedback,
		}

		_, err := pb.System.MsgPackageOrderApprove(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.MsgPackageOrderApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// @Summary 短信订单列表
// @Description 短信订单列表
// @Tags 短信订单
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true "第几页，1为起始页, -1 不分页返回所有"
// @Param page_size query int false "每页条数,-1 不分页返回所有"
// @Param status query int false "0不过滤,1待认证 2 已通过 3 未通过"
// @Success 200 {object} v1.MsgPackageOrderListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/order/msg_package [get]
func (c *Controller) MsgPackageOrderList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.MsgPackageOrderListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.MsgPackageOrderListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.MsgPackageOrderListResponse{}
		rpcReq := &v1.MsgPackageOrderListRequest{
			Status:     int64(req.Status),
			PageSize:   req.PageSize,
			Page:       req.Page,
			Management: true,
		}

		rpcRsp, err := pb.System.MsgPackageOrderList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.MsgPackageOrderList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.MsgPackageOrderItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
