// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"property-management-gateway/errors"
	param_v1 "property-management-gateway/param/v1"
	"property-management-gateway/pb"
	"property-management-gateway/pb/v1"
)

//
// @Summary 添加套餐
// @Description 添加套餐
// @Tags 套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.PackageAddBody true "信息"
// @Success 200 {object} v1.PackageAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/package [post]
func (c *Controller) PackageAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageAddRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.PackageAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageAddResponse{}
		rpcReq := &v1.PackageAddRequest{
			Name:       req.Name,
			Desc:       req.Desc,
			HouseCount: req.HouseCount,
			Price:      req.Price,
		}

		_, err := pb.Common.PackageAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.PackageAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 编辑套餐
// @Description 编辑套餐
// @Tags 套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.PackageUpdateBody true "信息"
// @Success 200 {object} v1.PackageUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/package [put]
func (c *Controller) PackageUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.PackageUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageUpdateResponse{}
		rpcReq := &v1.PackageUpdateRequest{
			Name:       req.Name,
			Desc:       req.Desc,
			HouseCount: req.HouseCount,
			Price:      req.Price,
			Id:         req.Id,
		}

		_, err := pb.Common.PackageUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.PackageUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 发布和下架
// @Description 发布和下架
// @Tags 套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.PackageEnableSetBody true "信息"
// @Success 200 {object} v1.PackageEnableSetResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/package/enable [put]
func (c *Controller) PackageEnableSet(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageEnableSetRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.PackageEnableSetBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageEnableSetResponse{}
		rpcReq := &v1.PackageEnableSetRequest{
			Enable: req.Enable,
			Id:     req.Id,
		}

		_, err := pb.Common.PackageEnableSet(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.PackageEnableSet"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 给套餐添加应用
// @Description 给套餐添加应用
// @Tags 套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.PackageApplicationSetBody true "信息"
// @Success 200 {object} v1.PackageApplicationSetResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/package/application [put]
func (c *Controller) PackageApplicationSet(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageApplicationSetRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.PackageApplicationSetBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageApplicationSetResponse{}
		rpcReq := &v1.PackageApplicationSetRequest{
			ApplicationIds: req.ApplicationIds,
			Id:             req.Id,
		}

		_, err := pb.Common.PackageApplicationSet(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.PackageApplicationSet"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除套餐
// @Description 删除套餐
// @Tags 套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true "套餐id"
// @Success 200 {object} v1.PackageDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/package [delete]
func (c *Controller) PackageDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.PackageDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageDelResponse{}
		rpcReq := &v1.PackageDelRequest{
			Id: req.Id,
		}

		_, err := pb.Common.PackageDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.PackageDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 套餐列表
// @Description 套餐列表
// @Tags 套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int true "第几页，1为起始页, -1 不分页返回所有"
// @Param page_size query int false "每页条数,-1 不分页返回所有"
// @Param enable query int false "0不过滤,1已发布的 2 未发布的"
// @Success 200 {object} v1.PackageListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/package [get]
func (c *Controller) PackageList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PackageListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.PackageListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.PackageListResponse{}
		rpcReq := &v1.PackageListRequest{
			//PackageId: req.PackageId,
			PageSize: req.PageSize,
			Page:     req.Page,
			Enable:   req.Enable,
		}

		rpcRsp, err := pb.Common.PackageList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.PackageList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.PackageItem, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 套餐的应用列表
// @Description 套餐的应用列表
// @Tags 套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param package_id query int true "套餐id"
// @Success 200 {object} v1.ApplicationListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/package/application [get]
func (c *Controller) PackageApplicationList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ApplicationListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationListResponse{}
		rpcReq := &v1.ApplicationListRequest{
			PackageId: req.PackageId,
			Page:      -1,
			PageSize:  -1,
		}

		rpcRsp, err := pb.Common.ApplicationList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.ApplicationItem, 0)
		}

		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 套餐还可添加的应用列表
// @Description 套餐还可添加的应用列表
// @Tags 套餐
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param package_id query int true "套餐id"
// @Success 200 {object} v1.ApplicationListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/package/application_can_add [get]
func (c *Controller) PackageApplicationCanAddList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ApplicationListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ApplicationListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		//tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		//if err != nil {
		//	return err
		//}
		// 响应数据
		resp := param_v1.ApplicationListResponse{}
		rpcReq := &v1.ApplicationListRequest{
			PackageId: req.PackageId,
			Page:      -1,
			PageSize:  -1,
		}

		rpcRsp, err := pb.Common.ApplicationList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		exclues := []int64{}
		for _, v := range rpcRsp.List {
			exclues = append(exclues, v.Id)
		}
		rpcReq = &v1.ApplicationListRequest{
			ExcludeIds: exclues,
			Page:       -1,
			PageSize:   -1,
		}

		rpcRsp, err = pb.Common.ApplicationList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.ApplicationList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.ApplicationItem, 0)
		}

		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
