// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"property-management-gateway/errors"
	param_v1 "property-management-gateway/param/v1"
	"property-management-gateway/pb"
	"property-management-gateway/pb/v1"
	"property-management-gateway/utils"
)

//
// @Summary 添加权限节点
// @Description 添加权限节点
// @Tags 物业权限节点
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.SystemPermissionAddBody true "节点信息"
// @Success 200 {object} v1.SystemPermissionAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/system_permission [post]
func (c *Controller) SystemPermissionAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.SystemPermissionAddRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.SystemPermissionAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.SystemPermissionAddResponse{}
		rpcReq := &v1.SystemPermissionAddRequest{
			Name:        req.Name,
			Pcode:       req.Pcode,
			Router:      req.Router,
			Path:        req.Path,
			AppletePath: req.AppletePath,
			Enable:      req.Enable,
		}

		rpcRsp, err := pb.Common.SystemPermissionAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.SystemPermissionAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:   ModulePermission,
			Action:   ActionPermissionAdd,
			Origin:   nil,
			Target:   req.SystemPermissionAddBody,
			UserName: tokenInfo.User,
			Uid:      tokenInfo.Uid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 更改权限节点
// @Description 更改权限节点
// @Tags 物业权限节点
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.SystemPermissionUpdateBody true "节点信息"
// @Success 200 {object} v1.SystemPermissionUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/system_permission [put]
func (c *Controller) SystemPermissionUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.SystemPermissionUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.SystemPermissionUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.SystemPermissionUpdateResponse{}
		rpcReq := &v1.SystemPermissionUpdateRequest{
			Name:        req.Name,
			Code:        req.Code,
			Path:        req.Path,
			AppletePath: req.AppletePath,
			Router:      req.Router,
		}

		rpcRsp, err := pb.Common.SystemPermissionUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.SystemPermissionUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:   ModulePermission,
			Action:   ActionPermissionUpdate,
			Origin:   rpcRsp.Origin,
			Target:   req.SystemPermissionUpdateBody,
			UserName: tokenInfo.User,
			Uid:      tokenInfo.Uid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除权限节点
// @Description 删除权限节点
// @Tags 物业权限节点
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param code query string true "节点code"
// @Success 200 {object} v1.SystemPermissionDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/system_permission [delete]
func (c *Controller) SystemPermissionDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.SystemPermissionDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.SystemPermissionDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.SystemPermissionDelResponse{}
		rpcReq := &v1.SystemPermissionDelRequest{
			Code: req.Code,
		}

		rpcRsp, err := pb.Common.SystemPermissionDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.SystemPermissionDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:   ModulePermission,
			Action:   ActionPermissionDel,
			Origin:   rpcRsp.Origin,
			Target:   nil,
			UserName: tokenInfo.User,
			Uid:      tokenInfo.Uid,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

func fixEmptySystemPermissionItem(list []*v1.SystemPermissionItem) []*v1.SystemPermissionItem {
	if list == nil {
		return make([]*v1.SystemPermissionItem, 0)
	}
	for i, v := range list {
		if v.Childs == nil {
			list[i].Childs = make([]*v1.SystemPermissionItem, 0)
			continue
		}
		list[i].Childs = fixEmptySystemPermissionItem(v.Childs)
	}
	return list
}

//
// @Summary 权限节点列表
// @Description 权限节点列表
// @Tags 物业权限节点
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.SystemPermissionListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/system_permission [get]
func (c *Controller) SystemPermissionList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.SystemPermissionListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.SystemPermissionListResponse{}
		rpcReq := &v1.SystemPermissionListRequest{}

		rpcRsp, err := pb.Common.SystemPermissionList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.SystemPermissionList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		rpcRsp.List = fixEmptySystemPermissionItem(rpcRsp.List)

		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 发布或下架
// @Description 发布或下架
// @Tags 物业权限节点
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.SystemPermissionEnableBody true "节点信息"
// @Success 200 {object} v1.SystemPermissionUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/system_permission/enable [put]
func (c *Controller) SystemPermissionEnable(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.SystemPermissionEnableRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.SystemPermissionEnableBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.SystemPermissionEnableResponse{}
		rpcReq := &v1.SystemPermissionEnableRequest{
			Code:   req.Code,
			Enable: req.Enable,
		}

		_, err := pb.Common.SystemPermissionEnable(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Common.SystemPermissionEnable"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
